from cherrytemplate import renderTemplate

import shutil, os, cStringIO, gzip, bz2, tarfile

from connection import conn
from sqlobject import *

from utils import *
import Image

from index import WebRoot


class Domain(SQLObject):
    _connection = conn
    _defaultOrder = "name" 
    name = StringCol()
    css = StringCol(default="")


class WebDomain(WebRoot):
    def index(self):
        return self.list()

    index = adminOnly(index)

    def list(self):
        domains = Domain.select()
        html = renderTemplate(file="templates/domains_list.html")
        return self.render(html)

    list = adminOnly(list)

    def add(self, name=None, css_filename=None, css=None, favicon=None):
        if name:
            domain = Domain(name=name)
            
            bad_file = False

            # Favicon
            if favicon.filename != "":
                ext = favicon.filename.split(".")[-1]
                
                if ext == "ico":
                    if not self._upload(name, favicon):
                        bad_file = True
                else:
                    bad_file = True

            # CSS File(s)
            if css_filename != "":
                ext = css_filename.split(".")[-1]
                
                if ext == "css" :
                    if css.filename != "" and self._upload(name, css, css_filename):
                        domain.css = css_filename
                    else:
                        bad_file = True
                else:
                    bad_file = True
            elif css_filename == "" and css.filename != "":
                bad_file = True

            if bad_file:
                return self.render_page("domain_upload_warning")

            raise cherrypy.HTTPRedirect("/domains")
        else:
            return self.render_page("admin_domain_add")

    add = adminOnly(add)

    def edit(self, did, name=None, css_filename=None, css=None, favicon=None): 
        domain = Domain.get(did)
        
        if name:
            domain.name = name

            bad_file = False
            
            if css_filename != "":
                domain.css = css_filename

            # Favicon
            if favicon.filename != "":
                ext = favicon.filename.split(".")[-1]
                
                if ext == "ico":
                    self.del_data(did, "fav")
                    if not self._upload(name, favicon):
                        bad_file = True
                else:
                    bad_file = True

            # CSS File(s)
            if css_filename != "":
                ext = css_filename.split(".")[-1]

                if ext == "css" :
                    if css.filename != ""\
                           and self.del_data(did, "css")\
                           and self._upload(name, css, css_filename):
                        domain.css = css_filename
                    elif not os.path.exists("css/"+domain.name):
                        print "HU"
                        bad_file = True
                else:
                    bad_file = True
            elif css_filename == "" and css.filename != "":
                bad_file = True
            else:
                domain.css = ""

            if bad_file:
                return self.render_page("domain_upload_warning")

            raise cherrypy.HTTPRedirect("/domains")
        else:
            html = renderTemplate(file="templates/admin_domain_edit.html")
            return self.render(html)

    edit = adminOnly(edit)

    def delete(self, did, confirmed=None):
        domain = Domain.get(did)
        
        if confirmed:
            # css path
            csspath = os.path.join("css", domain.name)
            if os.path.exists(csspath):
                remove_dir(csspath)
                
            # favicon path
            fav_name = domain.name.replace(".", "_") + ".ico"
            favpath = os.path.join("images", "favicon", fav_name)

            if os.path.exists(favpath) and os.path.isfile(favpath):
                os.unlink(favpath)
            
            Domain.delete(did)

            raise cherrypy.HTTPRedirect("/domains")
        else:
            conf_msg="Do you really want to delete domain %s ?" % (domain.name)
            conf_yes="delete"
            conf_hidden = { "did" : did,
                            "confirmed" : "yes" }
            
            html = renderTemplate(file="templates/confirm_action.html")
            
            return self.render(html) 

    delete = adminOnly(delete)

    def del_data(self, did, data, ref=None):
        domain = Domain.get(did)

        if data == "css":
            csspath = os.path.join("css", domain.name)
            if os.path.exists(csspath):
                remove_dir(csspath)
        
        elif data == "fav":
            fav_filename = domain.name.replace(".", "_") + ".ico"
            dstname = os.path.join("images", "favicon", fav_filename)
            
            if os.path.exists(dstname) and os.path.isfile(dstname):
                os.unlink(dstname)

        if ref:
            raise cherrypy.HTTPRedirect(ref)

        return True

    del_data = adminOnly(del_data)

    def _upload(self, name, domain, css_filename=""):
        # css case
        if css_filename != "":
            # css directory path
            csspath = os.path.join("css", name)
            if not os.path.exists(csspath):
                os.mkdir(csspath)

            ext1 = domain.filename.split(".")[-1]
            ext2 = domain.filename.split(".")[-2]

            # Not compressed
            if ext1 == "css":
                data = self._read_data(domain.file)
                self._write_file(csspath, css_filename, data)
                
            # Compressed
            elif ext1 == "gz" or ext1 == "bz2":
                try:
                    # Gzip
                    if ext1 == "gz":
                        zfile = gzip.GzipFile(mode='rb', fileobj=domain.file)
                        data = self._read_data(zfile)
                    
                    # Bzip2
                    else:
                        zfile = self._read_data(domain.file)
                        data = bz2.decompress(zfile)

                    # Tarball
                    if ext2 == "tar":
                        self._decompress_tar(csspath, data)
                    else:
                        self._write_file(csspath, css_filename, data)
                except Exception:
                    return False
    
            # Extention not supported
            else:
                return False

            return True
        
        else:
            # read stream
            data = self._read_data(domain.file)
                
            # path...
            favpath = os.path.join("images", "favicon")
            fav_filename = name.replace(".", "_") + ".ico"

            # write to file
            self._write_file(favpath, fav_filename, data)

            return True


    def _read_data(self, fileobj):
        data = fileobj.read(1024 * 8)
        while 1:
            tmp = fileobj.read(1024 * 8)
            if not tmp: break
            data += tmp
        
        return data
    
    def _write_file(self, path, filename, data):
        dstname = os.path.join(path, filename)

        # write file
        f = open(dstname, 'wb')
        f.write(data)
        f.close()

    def _decompress_tar(self, path, data):
        # paths
        csspath = path
        tmppath = os.path.join("css", "tmp")
        if not os.path.exists(tmppath):
            os.mkdir(tmppath)

        # extract files
        tardata = cStringIO.StringIO(data)
        tar = tarfile.open(mode='r:', fileobj=tardata)

        members = tar.getnames()
        for el in members:
            tar.extract(el, tmppath)
            
        # list files
        files = os.listdir(tmppath)
        nb = len(files)

        # move to css directory
        if nb == 1:
            dirpath = os.path.join(tmppath, files[0])
            shutil.move(dirpath, csspath)
            os.rmdir(tmppath)
        else:
            shutil.move(tmppath, csspath)

        tar.close()
